Package: forestFloor
Type: Package
Title: Visualizes Random Forests with Feature Contributions
Version: 1.11.1
Date: 2017-05-30
Author: Soeren Havelund Welling
Maintainer: Soeren Havelund Welling <SORHAWELL@GMAIL.COM>
Depends:
Suggests: caret, e1071
Description: Form visualizations of high dimensional mapping structures of random forests and feature contributions.
SystemRequirements: OpenGL, GLU Library, zlib
License: GPL-2
URL: http://forestFloor.dk
Imports: Rcpp (>= 0.11.3), rgl, kknn, randomForest
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2017-05-30 10:38:49 UTC; sowe
Repository: CRAN
Date/Publication: 2017-05-30 12:17:27 UTC
Built: R 4.1.0; x86_64-w64-mingw32; 2021-05-28 05:24:58 UTC; windows
Archs: x64
