useDynLib(forestFloor)

importFrom(randomForest,importance)

importFrom(Rcpp, evalCpp)
import(kknn)

importFrom(stats,
           cor,
           prcomp,
           predict)

#graphicals
importFrom(graphics,
           par,
           plot,
           points,
           segments)

importFrom(grDevices,
           col2rgb,
           hsv,
           rgb,
           rgb2hsv
           )

importFrom(stats,
           delete.response,
           model.frame,
           na.fail,
           na.omit)

importFrom(utils,
           capture.output
)

import(rgl)

#methods
#plotting
S3method(plot, forestFloor_regression)
S3method(plot, forestFloor_multiClass)
#printing
S3method(print,forestFloor_regression)
S3method(print,forestFloor_multiClass)
#3d images
S3method(show3d,  forestFloor_multiClass)
S3method(show3d,  forestFloor_regression)



export(
       append.overwrite.alists,
       box.outliers,
       convolute_ff,
       convolute_ff2,
       convolute_grid,
       fcol,
       forestFloor,
       plot_simplex3,
       recTree,
       show3d,
       vec.plot,
       Xtestmerger,
       importanceExportWrapper
       )
